"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const fs = require("fs-extra");
const path = require("path");
const run_1 = require("../../base/run");
const docker_run_1 = require("../../core/docker.run");
const conf_1 = require("../../services/conf");
const logger_1 = require("../../services/logger");
const rdehook_1 = require("../../services/rdehook");
const sync_1 = require("../../services/sync");
class DockerRun extends run_1.default {
    async preInit() {
        await super.preInit();
        const { flags } = this.parse(DockerRun);
        this.extras = flags.extras;
    }
    async initialize() {
        await super.initialize();
        if (this.useLocalFlag) {
            logger_1.debug('using local flag');
            await sync_1.default.start({
                watch: this.watch,
                cmd: this.cmd,
                skipInstall: true
            });
            conf_1.default.useLocal = this.useLocal;
        }
    }
    async preRun() {
        const core = new docker_run_1.default({
            watch: this.watch,
        });
        await core.start();
        await rdehook_1.default.trigger('preRun');
    }
    async run() {
        if (!this.cmd) {
            return;
        }
        process.env.PATH = `${process.env.PATH}:${conf_1.default.cwd}/node_modules/.bin`;
        let args = ['run', `${this.cmd}`];
        if (this.extras) {
            args.push('--');
            let extras = [];
            if (this.cmd === 'lint') {
                extras = this.extras.split(' ').map(item => {
                    // fix path to relative to .rde path
                    if (/^(app|template)\/.*/.test(item)) {
                        return path.posix.join('..', item);
                    }
                    return item;
                });
            }
            else {
                extras = this.extras.split(' ');
            }
            args = args.concat(extras);
        }
        if (process.platform === 'win32') {
            await fs.ensureSymlink('node_modules', path.join('.integrate', 'node_modules'));
        }
        process.on('SIGINT', () => {
            child.kill('SIGINT');
            process.exit(0);
        });
        let child = child_process_1.spawn('npm', args, {
            cwd: conf_1.default.isIntegrate ? conf_1.default.integrateDir : conf_1.default.runtimeDir,
            env: process.env,
            stdio: 'inherit',
            shell: true
        });
        child.on('close', code => {
            child.kill('SIGINT');
            process.exit(code);
        });
        await rdehook_1.default.trigger('postRun');
    }
}
DockerRun.description = 'run script inside docker container';
DockerRun.examples = [
    '$ rde docker:run <cmd>',
];
DockerRun.args = [...run_1.default.args];
DockerRun.flags = Object.assign({}, run_1.default.flags);
exports.default = DockerRun;
