"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const cache_1 = require("../../services/cache");
const conf_1 = require("../../services/conf");
const docker_1 = require("../../services/docker");
const ide_1 = require("../../services/ide");
const render_1 = require("../../services/render");
const sync_1 = require("../../services/sync");
const util_1 = require("../../util");
const index_1 = require("./index");
const { join, resolve } = path;
class ApplicationCreate extends index_1.default {
    constructor() {
        super(...arguments);
        this.rdcConf = null;
    }
    async prepare() {
        await docker_1.default.pull(this.rdc);
        await util_1.default.asyncExec(`mkdir ${conf_1.default.localCacheDir}`);
        cache_1.default.set('container', this.rdc);
        const { cwd, rdcConfName, dockerWorkDirRoot, } = conf_1.default;
        const rdcConfPath = resolve(conf_1.default.localCacheDir, rdcConfName);
        await docker_1.default.copy(this.rdc, [{
                // don't use join or resolve, cuz windows will change / to C:/
                from: `${dockerWorkDirRoot}/app`,
                to: resolve(cwd, 'app'),
            }]);
        await sync_1.default.genAppStagedFiles(this.rdc);
        await sync_1.default.install();
        await ide_1.default.initSettings(true);
        this.rdcConf = require(rdcConfPath);
    }
    async genConfFile() {
        const { appConfName } = conf_1.default;
        const { docs, docker = { ports: [] } } = this.rdcConf;
        await render_1.default.renderTo(join('rda', appConfName.slice(0, -3)), {
            container: this.rdc,
            docs: docs ? docs.url : '',
            ports: docker.ports,
        }, appConfName);
    }
    async genExtraFiles() {
        await render_1.default.renderTo(join('rda', 'README'), {
            name: this.name,
            homepage: conf_1.default.homepage,
        }, 'README.md');
        await render_1.default.renderTo('.gitignore', {
            isApp: conf_1.default.isApp
        }, '.gitignore', {
            overwrite: true,
        });
        await ide_1.default.initSettings(true);
    }
}
exports.default = ApplicationCreate;
