"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const util_1 = require("../util");
const conf_1 = require("./conf");
const logger_1 = require("./logger");
const render_1 = require("./render");
class Docker {
    async pull(image) {
        await util_1.default.asyncExec(`docker pull ${image}`);
    }
    async copy(image, mappings) {
        const containerName = `rde-dummy-${+new Date()}`;
        if (await this.containerExist(containerName)) {
            await util_1.default.asyncExec(`docker container rm ${containerName}`);
        }
        await util_1.default.asyncExec(`docker create -it --name ${containerName} ${image} bash`);
        for (const mapping of mappings) {
            await util_1.default.asyncExec(`docker cp ${containerName}:${mapping.from} ${mapping.to}`);
        }
        await util_1.default.asyncExec(`docker rm -fv ${containerName}`);
    }
    async imageExist(name) {
        try {
            await util_1.default.asyncExec(`docker inspect --type=image ${name}`);
            return true;
        }
        catch (e) {
            if (e) { }
            return false;
        }
    }
    async containerExist(name) {
        try {
            await util_1.default.asyncExec(`docker ps -a | grep ${name}`);
            return true;
        }
        catch (e) {
            if (e) { }
            return false;
        }
    }
    async build(tag, cwd, noCache = false, dockerFile = 'Dockerfile', context = '.') {
        let args = ['build', '-t', tag, '-f', dockerFile, context];
        if (noCache) {
            args.splice(1, 0, '--no-cache');
        }
        logger_1.debug(`cwd ${cwd}`);
        logger_1.debug(`docker ${args.join(' ')}`);
        await util_1.default.asyncSpawn('docker', args, {
            cwd,
        });
        await util_1.default.asyncExec('rm -f .dockerignore');
        await util_1.default.asyncExec('docker image prune -f');
    }
    async tag(oldtag, newtag) {
        await util_1.default.asyncSpawn('docker', ['tag', oldtag, newtag]);
    }
    async push(name, version) {
        await util_1.default.asyncSpawn('docker', ['push', `${name}:${version}`]);
        await util_1.default.asyncSpawn('docker', ['push', `${name}:latest`]);
        await util_1.default.asyncExec('rm -f .dockerignore');
    }
    async genDockerFile(workDir, from, dir, isApp) {
        // gen dockerfile， 不生成dockerignore
        await render_1.default.renderTo('docker/.dockerignore', {}, '.dockerignore', {
            overwrite: true,
        });
        let uid = null;
        if (process.getuid) {
            uid = process.getuid();
        }
        await render_1.default.renderTo('docker/Dockerfile', {
            from,
            workDir,
            dockerWorkDirRoot: conf_1.default.dockerWorkDirRoot,
            isApp,
            uid,
        }, `${dir}/Dockerfile`, {
            overwrite: true,
        });
    }
    async genDockerFile4Publish(workDir, from, dir) {
        // gen dockerfile
        await render_1.default.renderTo('docker/.dockerignore', {}, '.dockerignore', {
            overwrite: true,
        });
        await render_1.default.renderTo('docker/Dockerfile.publish', {
            from,
            workDir,
        }, `${dir}/Dockerfile`, {
            overwrite: true,
        });
    }
    async genDockerCompose(workDir, cmd, ports, watch, tag, dir, isApp = false, isBuild = false) {
        if (!tag) {
            const name = path.basename(conf_1.default.cwd);
            tag = `${name}:latest`;
        }
        await render_1.default.renderTo('docker/docker-compose', {
            workDir,
            cmd,
            ports,
            watch,
            tag,
            isApp,
            isBuild,
        }, `${dir}/docker-compose.yml`, {
            overwrite: true,
        });
    }
}
tslib_1.__decorate([
    logger_1.default('Pulling image from docker hub...')
], Docker.prototype, "pull", null);
tslib_1.__decorate([
    logger_1.default('Copying files from image...')
], Docker.prototype, "copy", null);
tslib_1.__decorate([
    logger_1.default('Pushing image to docker hub...')
], Docker.prototype, "push", null);
tslib_1.__decorate([
    logger_1.default('Generating docker files...')
], Docker.prototype, "genDockerFile", null);
tslib_1.__decorate([
    logger_1.default('Generating Dockerfile...')
], Docker.prototype, "genDockerFile4Publish", null);
tslib_1.__decorate([
    logger_1.default('Generating docker-compose.yml...')
], Docker.prototype, "genDockerCompose", null);
exports.default = new Docker();
